#!/usr/bin/ruby -w

require '../lib/gff.rb';
require '../lib/gmm.rb';

filename_species_conf = '../etc/celegans.yaml';

## For DE genes
#filename_idx = '../data/celegans/gene_index.WS150';
#filename_data = '../local/ABdown0.01.tab';
#filename_output = '../data/celegans/ABdown0.01';

## For RNAi
#filename_idx = '../data/celegans/pcr_product_index.WS150';
filename_idx = '../data/celegans/gene_index.WS150';
filename_data = '../local/RNAi.tab';
filename_output = '../data/celegans/RNAi';

bin_size = 100;
bin_unit = 1000;
gm = GenomeMap.new(bin_size, bin_unit);

idx_data = gm.load_gm_index(filename_idx);
list_data = Hash.new();
file_data = File.new(filename_data,'r');
file_data.each_line{ |line|
    line.chomp!();
    tmp = line.split(/\s+/);
    reagent_name = tmp[0];
    list_data[reagent_name] = 1 if( reagent_name =~ /[A-Za-z]/ );
}

list_gm = gm.list_to_gm_data( list_data, idx_data );
conf_species = gm.load_conf(filename_species_conf);

max_up = 0.0; min_down = 0.0;
output_strings = Array.new();
conf_species['chromosome'].each { |tmp_chr|
    chr_length = conf_species['chromosome_length'][tmp_chr].to_i;
    num_bins = chr_length / bin_size / bin_unit;
    gm_chr = list_gm[tmp_chr];
    
    for bin_pos in 0..num_bins
        if( gm_chr.bins[bin_pos] == nil )
            gm_chr.bins[bin_pos] = GenomeMapBin.new();
            gm_chr.bins[bin_pos].up = 0.0; 
            gm_chr.bins[bin_pos].down = 0.0;
        end
        output_string = "#{tmp_chr}\t#{bin_pos*bin_size*bin_unit}\t"+
                        "#{gm_chr.bins[bin_pos].up}\t"+
                        "#{gm_chr.bins[bin_pos].down}";
        output_strings << output_string;
        if( gm_chr.bins[bin_pos].up > max_up )
            max_up = gm_chr.bins[bin_pos].up;
        end

        if( gm_chr.bins[bin_pos].down < min_down )
            min_down = gm_chr.bins[bin_pos].down;
        end
    end
}

$stderr.puts "Write #{filename_output}";
file_output = File.new(filename_output, 'w');
file_output.puts "MAX\t#{max_up}";
file_output.puts "MIN\t#{min_down}";
output_strings.each{ |tmp_output| file_output.puts tmp_output; }
