#!/usr/bin/ruby -w

require '../lib/gff.rb';
require '../lib/gmm.rb';

dir_wormdata = '../../wormdata/';
filename_species_conf = '../etc/celegans.yaml';

gff = GFF.new();

## For gene
filename_gff = dir_wormdata+'WS150/elegansWS150.CDS.gff';
tmpl_output = '../data/celegans/gene.%d.WS150';
gff.source_condition = 'curated';
gff.attribute_condition = 'WBGene';

## For repeat_all
=begin
filename_gff = dir_wormdata+'WS150/elegansWS150.repeat_all.gff';
tmpl_output = '../data/celegans/repeat_all.%d.WS150';
gff.source_condition = nil;
gff.attribute_condition = nil;
=end

genes_gff = gff.load_gff( filename_gff );

bin_size = 100;
bin_unit = 1000;
gm = GenomeMap.new(bin_size, bin_unit);
genes_gm = gm.gff_to_gm_data( genes_gff );
conf_species = gm.load_conf(filename_species_conf);

max_up = 0.0; min_down = 0.0;
output_strings = Array.new();
conf_species['chromosome'].each { |tmp_chr|
    chr_length = conf_species['chromosome_length'][tmp_chr].to_i;
    num_bins = chr_length / bin_size / bin_unit;
    gm_chr = genes_gm[tmp_chr];
    
    for bin_pos in 0..num_bins
        if( gm_chr.bins[bin_pos] == nil )
            gm_chr.bins[bin_pos] = GenomeMapBin.new();
            gm_chr.bins[bin_pos].up = 0.0; 
            gm_chr.bins[bin_pos].down = 0.0;
        end
        output_string = "#{tmp_chr}\t#{bin_pos*bin_size*bin_unit}\t"+
                        "#{gm_chr.bins[bin_pos].up}\t"+
                        "#{gm_chr.bins[bin_pos].down}";
        output_strings << output_string;
        if( gm_chr.bins[bin_pos].up > max_up )
            max_up = gm_chr.bins[bin_pos].up;
        end

        if( gm_chr.bins[bin_pos].down < min_down )
            min_down = gm_chr.bins[bin_pos].down;
        end
    end
}

filename_output = sprintf(tmpl_output, bin_size);
$stderr.puts "Write #{filename_output}";
file_output = File.new(filename_output, 'w');
file_output.puts "MAX\t#{max_up}";
file_output.puts "MIN\t#{min_down}";
output_strings.each{ |tmp_output| file_output.puts tmp_output; }
