#!/usr/bin/ruby -w

require '../lib/gmm.rb';
require '../lib/svg.rb';

filename_gm_conf = '../etc/gmm.yaml';
filename_species_conf = '../etc/celegans.yaml';
bin_size = 100;
bin_unit = 1000;

gm = GenomeMap.new(bin_size, bin_unit);
gm_conf = gm.load_conf(filename_gm_conf);
sp_conf = gm.load_conf(filename_species_conf);

labels = { 
    'up' => 'ABup (P &lt; 0.01)', 'down' => 'ABdown (P &lt; 0.01)',
    'rnai' => 'RNAi',
    'gene' => 'All genes', 'repeat' => 'All repeats' };

files = {
    'up' => '../data/celegans/ABup0.01_100.WS150',
    'down' => '../data/celegans/ABdown0.01_100.WS150',
    'rnai' => '../data/celegans/RNAi_100.WS150',
    'gene' => '../data/celegans/gene_100.WS150',
    'repeat' => '../data/celegans/repeat_all_100.WS150' };

bg_style = {'up' => 'fill:pink', 'down' => 'fill:lightgreen',
            'rnai' => 'fill:lightblue', 'gene' => 'fill:lightblue', 
            'repeat' => 'fill:lightblue' };

data_style = {'up' => 'fill:darkred', 'down' => 'fill:darkgreen',
            'rnai' => 'fill:darkblue', 'gene' => 'fill:darkblue', 
            'repeat' => 'fill:darkblue' };

data_order = ['up','down','rnai','gene','repeat'];
#data_order = ['up'];

gm_data = Hash.new();
data_order.each{ |tmp_data|
    $stderr.puts "Load #{files[tmp_data]}";
    gm_data[tmp_data] = gm.load_gm_data( files[tmp_data] );
}

figure_width = sp_conf['chromosome_length']['longest'].to_i;
figure_width = figure_width / gm_conf['pixel_unit'];
figure_height = gm_conf['chr_box']['height'] * data_order.length *
                    sp_conf['chromosome'].length +
                gm_conf['chr_box']['y_chr_margin'] * 
                    (sp_conf['chromosome'].length - 1) +
                gm_conf['chr_box']['y_box_margin'] *
                    (sp_conf['chromosome'].length * (data_order.length-1)) +
                gm_conf['y_top_margin'] + gm_conf['y_bottom_margin'];

bar_width = bin_unit * bin_size / gm_conf['pixel_unit'];

svg     = SVG.new();
chr_box = SVGRect.new();
bar     = SVGRect.new();
text    = SVGText.new();

puts svg.header(figure_width, figure_height);

chr_idx = 0;
sp_conf['chromosome'].each{ |tmp_chr|
    data_idx = 0;
    num_bins = sp_conf['chromosome_length'][tmp_chr].to_i / bin_size / bin_unit;

    data_order.each{ |tmp_data|
        box_width = sp_conf['chromosome_length'][tmp_chr].to_i;
        box_width = box_width / gm_conf['pixel_unit'];
        box_height = gm_conf['chr_box']['height'];
        box_x = gm_conf['chr_box']['x'];
        box_y = gm_conf['y_top_margin'];
        box_y += gm_conf['chr_box']['height']*data_order.length*chr_idx; 
        box_y += gm_conf['chr_box']['y_box_margin']*data_order.length*
                (chr_idx-1);
        box_y += gm_conf['chr_box']['y_chr_margin']*chr_idx
        box_y += gm_conf['chr_box']['height']*data_idx;
        box_y += gm_conf['chr_box']['y_box_margin']*data_idx;
        chr_box.set_pos(box_x,box_y);
        chr_box.set_size(box_width, box_height);
        chr_box.style = bg_style[tmp_data];
        puts chr_box.draw();

        for bin_pos in 0..num_bins
            bar_height = gm_data[tmp_data][tmp_chr].bins[bin_pos].up /
                        (gm_data[tmp_data][tmp_chr].max -
                            gm_data[tmp_data][tmp_chr].min) *
                        gm_conf['chr_box']['height'];
            bar_x = box_x + (bin_pos * bar_width);
            bar_y = box_y + gm_conf['chr_box']['height'] - bar_height;
            bar.set_pos(bar_x, bar_y);

            bar_width_adjusted = bar_width;
            if( bar_x + bar_width > box_x + box_width )
                bar_width_adjusted = box_width + box_x - bar_x;
            end

            bar.set_size(bar_width_adjusted, bar_height);
            bar.style = data_style[tmp_data];
            puts bar.draw();
        end
        data_idx += 1;

        ## Chromosome label
        chr_label_x = gm_conf['chr_label']['x'];
        chr_label_y = box_y + gm_conf['chr_label']['font_size'];
        text.set_pos(chr_label_x, chr_label_y);
        text.set_font(gm_conf['font'], gm_conf['chr_label']['font_size']);
        text.content = 'Chr '+tmp_chr;
        puts text.draw();

        ## Data label
        data_label_x = gm_conf['data_label']['x'];
        data_label_y = box_y + gm_conf['data_label']['font_size'];
        data_label_y += gm_conf['chr_label']['font_size'];
        text.set_pos(data_label_x, data_label_y);
        text.set_font(gm_conf['font'], gm_conf['data_label']['font_size']);
        text.content = labels[tmp_data];
        puts text.draw();

        ## Value label
        value_label_x = box_x + box_width + gm_conf['value_label']['x_margin'];
        value_label_y = box_y + gm_conf['value_label']['font_size'];
        text.set_pos(value_label_x, value_label_y);
        text.set_font(gm_conf['font'], gm_conf['value_label']['font_size']);
        text.content = gm_data[tmp_data][tmp_chr].max.to_s;
        puts text.draw();
    }
    chr_idx += 1;

    
}

puts svg.footer();
