#!/usr/bin/ruby -w
require 'yaml';

class GenomeMapBin
    attr_accessor :up, :down;
    
    def initialize
        @up = 0.0; @down = 0.0;
    end
    
    def add_bin_value( arg )
        @up += arg if( arg > 0 );
        @down -= arg if( arg < 0 );
    end
end

class GenomeMapIndex
    attr_accessor :chromosome, :position;

    def initialize(arg_chr, arg_pos)
        @chromosome = arg_chr; @position = arg_pos;
    end
end

class GenomeMapChr
    attr_accessor :max, :min;
    attr_reader :bins;

    def initialize
        @bins = Array.new();
    end

    def add_bin(arg_pos, arg_value)
        if(@bins[arg_pos] == nil)
            @bins[arg_pos] = GenomeMapBin.new() 
        end
        @bins[arg_pos].add_bin_value( arg_value );
    end
end

class GenomeMap
    attr_accessor :bin_size, :bin_unit;

    def initialize(arg_bin_size, arg_bin_unit)
        @bin_size = arg_bin_size; @bin_unit = arg_bin_unit;
    end
    
    def load_conf( arg_filename )
        return YAML.load( File.open(arg_filename) );
    end

    def load_gm_data(filename_data)
        rv = Hash.new();

        file_data = File.new(filename_data);
        max_up = file_data.gets(); max_up.chomp!(); 
        min_down = file_data.gets(); min_down.chomp!(); 
        max_up.sub!(/MAX\s+/,''); min_down.sub!(/MIN\s+/,'');

        file_data.each_line{ |line|
            line.chomp!();
            (tmp_chr, tmp_pos, tmp_up, tmp_down) = line.split(/\t/);
            rv[tmp_chr] = GenomeMapChr.new() if(not rv.has_key?( tmp_chr ));
            tmp_pos = tmp_pos.to_i / @bin_size / @bin_unit;
            rv[tmp_chr].add_bin(tmp_pos.to_i, tmp_up.to_f);
            rv[tmp_chr].add_bin(tmp_pos.to_i, tmp_down.to_f);
            rv[tmp_chr].max = max_up.to_f;
            rv[tmp_chr].min = min_down.to_f;
        }
        file_data.close();

        return rv;
    end

    def load_gm_index(filename_idx)
        rv = Hash.new();
        file_idx = File.new(filename_idx);
        file_idx.each_line{ |line|
            line.chomp!();
            (tmp_key, tmp_chr, tmp_pos) = line.split(/\t/);
            if(not rv.has_key?( tmp_key ) )
                rv[tmp_key] = GenomeMapIndex.new(tmp_chr, tmp_pos);
            end
        }
        file_idx.close();
        
        return rv;
    end

    ## Argument : list_data - hash (key = key of idx_data, value = measure)
    ##            idx_data - output of load_gm_index()
    def list_to_gm_data( list_data, idx_data )
        rv = Hash.new();
        list_data.each{ |tmp_item, tmp_value|
            if( idx_data.has_key?(tmp_item) )
                chromosome = idx_data[tmp_item].chromosome;
                position = idx_data[tmp_item].position;
            elsif( idx_data.has_key?(tmp_item+'a') )
                chromosome = idx_data[tmp_item+'a'].chromosome;
                position = idx_data[tmp_item+'a'].position;
            else
                $stderr.puts 'No index info : '+tmp_item;
            end
            
            if(not rv.has_key?( chromosome ))
                rv[chromosome] = GenomeMapChr.new();
            end
            
            position = position.to_i / @bin_unit.to_i / @bin_size.to_i;
            rv[chromosome].add_bin(position, tmp_value);
        }

        return rv;
    end

    ## Argument : array of GFFElement instances, which can be made by
    ##              load_gff() at GFF class (located at 'gff.rb')
    def gff_to_gm_data( gff_array )
        rv = Hash.new();
        gff_array.each{ |tmp_gff|
            gm_pos = tmp_gff.start.to_i / @bin_unit.to_i / @bin_size.to_i;
            chromosome = tmp_gff.seqname;
            
            if(not rv.has_key?( chromosome ))
                rv[chromosome] = GenomeMapChr.new();
            end
            
            #$stderr.puts tmp_gff.start if( chromosome == 'I' && gmm_pos == 1 );
            rv[chromosome].add_bin(gm_pos, 1);
        }
        return rv;
    end
end
