#!/usr/bin/ruby -w

class SVGObject
    attr_accessor :pos_x, :pos_y, :style;
    
    def initialize
        @pos_x = nil; @pos_y = nil; @style = nil;
    end

    def set_pos(tmp_x, tmp_y)
        @pos_x = tmp_x; @pos_y = tmp_y;
    end

    def add_style(arg_string)
        arg_string += ' style="'+@style+'"' if( @style != nil );
        return arg_string;
    end
end

class SVGText < SVGObject
    attr_accessor :font, :size, :content;

    def set_font(arg_font, arg_size)
        @font = arg_font; @size = arg_size;
    end

    def draw
        return '<text x="'+@pos_x.to_s+'" y="'+@pos_y.to_s + '" ' +
            'font-family="'+@font+'" font-size="'+@size.to_s+'">' +
            @content+'</text>';
    end
end

class SVGRect < SVGObject
    attr_accessor :width, :height;
    
    def set_size(arg_width, arg_height)
        @width = arg_width; @height = arg_height;
    end

    def draw
        rv = '<rect x="'+@pos_x.to_s+'" y="'+@pos_y.to_s + '" ' +
            ' width="'+@width.to_s+'" height="'+@height.to_s+'"';
        rv = add_style(rv);
        rv += '/>';
    end
end

class SVG
    attr_accessor :encoding;

    def initialize
        @encoding = 'UTF-8';
    end

    def http_header
        return "Content-type: image/svg+xml\r\n\r\n";
    end

    def header(width, height)
        return '<?xml version="1.0" encoding="'+@encoding+
            '" standalone="no"?>'+"\n"+
            '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 20001102//EN"'+"\n"+
            '"http://www.w3.org/TR/2000/CR-SVG-20001102/DTD/svg-20001102.dtd">'+
            "\n"+'<svg width="'+width.to_s+'" height="'+height.to_s+'" '+
            'version="1.1" xmlns="http://www.w3.org/2000/svg">';
    end

    def footer
        return '</svg>';
    end

    def comment(comment_text)
        return '<!--'+comment_text+'-->';
    end
end
